// Blending: A sphere inside a cube
#include <GL/glut.h>
#include <stdlib.h>
#include <stdio.h>

static float solidZ = 0.0;

void init(void)
{
    GLfloat mat_specular[] = {1.0,1.0,1.0,0.15};
    GLfloat mat_shininess[] = {100.0};
    GLfloat light0_position[] = {0.5,0.5,1.0,0.0};
    
    glClearColor (0.1, 0.1, 0.2, 0.0);
    glShadeModel (GL_SMOOTH);    
    
    glMaterialfv (GL_FRONT, GL_SPECULAR, mat_specular);
    glMaterialfv (GL_FRONT, GL_SHININESS, mat_shininess);
    glLightfv (GL_LIGHT0, GL_POSITION, light0_position);
    
    glEnable (GL_LIGHTING);
    glEnable (GL_LIGHT0);
    glEnable (GL_DEPTH_TEST);
}
    
void display (void)
{ 
    GLfloat mat_solid[] = {0.75, 0.75, 0.0, 1.0};
    GLfloat mat_zero[] = {0.0,0.0,0.0,1.0};
    GLfloat mat_transparent[] = {0.0,0.8,0.8,0.8};
    GLfloat mat_emission[] = {0.0,0.3,0.3,0.6};

    glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   
    glPushMatrix ();
        glMaterialfv(GL_FRONT,GL_EMISSION,mat_zero);
        glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_solid);
        glutSolidSphere (0.4, 80, 64);
    glPopMatrix ();
    
    glPushMatrix ();            
        glTranslatef (0.1, 0.1, solidZ);
        glRotatef (15.0, 1.0, 1.0, 0.0);
        glRotatef (30.0, 0.0, 1.0, 0.0);
        glMaterialfv(GL_FRONT,GL_EMISSION,mat_emission);
        glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_transparent);
        glEnable(GL_BLEND);
        glDepthMask(GL_FALSE);
        glBlendFunc(GL_SRC_ALPHA,GL_ONE);
        glutSolidCube (1.0);
        glDepthMask(GL_TRUE);
        glDisable(GL_BLEND);
    glPopMatrix ();
    
    glutSwapBuffers ();
}

void reshape(int w, int h)
{
    glViewport (0, 0, (GLsizei) w, (GLsizei) h);
    glMatrixMode (GL_PROJECTION);
    glLoadIdentity ();
    if (w <= h)
        glOrtho (-1.5, 1.5, -1.5*(GLfloat)h/(GLfloat)w, 
        1.5*(GLfloat)h/(GLfloat)w, -10.0, 10.0);
    else
        glOrtho (-1.5*(GLfloat)w/(GLfloat)h, 
        1.5*(GLfloat)w/(GLfloat)h,-1.5,1.5, -10.0, 10.0);
    glMatrixMode (GL_MODELVIEW);
    glLoadIdentity ();
}

void animate (void)
{
    solidZ -= 0.0006;
    if (solidZ <= -5.0) solidZ = 5.0;
    glutPostRedisplay();
}

void keyboard (unsigned char key, int x, int y)
{
    switch (key) {
        case 'y':
            glutIdleFunc (animate);
            break;
        case 'n':
            glutIdleFunc (NULL);
            break;
        default:
            break;
    }
}

int main(int argc, char** argv)
{
    glutInit (&argc, argv);
    glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
    glutInitWindowSize (1000, 1000);
    glutInitWindowPosition (100, 100);
    glutCreateWindow ("Blending: A sphere inside a cube");
    init ();
    glutDisplayFunc (display);
    glutReshapeFunc (reshape);
    glutKeyboardFunc (keyboard);
    glutMainLoop();
    return 0;
}
