//  Solar system
#include <GL/glut.h>
#include <stdlib.h>

static GLfloat year = 0.0, day = 0.0;

void init(void)
{
    GLfloat mat_specular[] = {1.0,1.0,1.0,0.15};
    GLfloat mat_shininess[] = {100.0};
    GLfloat light0_diffuse[] = {1.0,1.0,1.0,1.0};
    GLfloat light0_position[] = {1.0,1.0,1.0,0.0};
    
    glClearColor (0.1, 0.1, 0.2, 0.0);
    glShadeModel (GL_SMOOTH);    
    
    glMaterialfv (GL_FRONT, GL_SPECULAR, mat_specular);
    glMaterialfv (GL_FRONT, GL_SHININESS, mat_shininess);
    glLightfv (GL_LIGHT0, GL_DIFFUSE, light0_diffuse);
    glLightfv (GL_LIGHT0, GL_POSITION, light0_position);
    
    glEnable (GL_LIGHTING);
    glEnable (GL_LIGHT0);
    glEnable (GL_DEPTH_TEST);
}
    
void display (void)
{ 
    GLfloat mat_solid[] = {0.9, 0.75, 0.0, 1.0};
    GLfloat mat_zero[] = {0.0,0.0,0.0,1.0};
    GLfloat mat_transparent[] = {0.0,0.8,0.8,0.6};
    GLfloat mat_emission[] = {0.9,0.1,0.0,0.8};

    glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glColor3f (1.0, 1.0, 1.0);
    glPushMatrix ();
        glRotatef (year, 0.0, 1.0, 0.0);
        glMaterialfv(GL_FRONT,GL_EMISSION,mat_emission);
        glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_transparent);
        glutSolidSphere (1.0, 80, 64);
        
        glTranslatef (2.0, 0.0, 0.0);
        glRotatef (day, 0.0, 1.0, 0.0);
        glMaterialfv(GL_FRONT,GL_EMISSION,mat_zero);
        glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_solid);
        glEnable(GL_BLEND);
        glDepthMask(GL_FALSE);
        glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
        glutSolidSphere (0.3, 160, 160);
        glDepthMask(GL_TRUE);
        glDisable(GL_BLEND);

    glPopMatrix ();
    glutSwapBuffers ();
}

void yearDisplay (void)
{
    year = year + 0.2; 
    if (year > 360.0)
        year = year - 360.0;
    day = day + 0.5; 
    if (day > 360.0)
        day = day - 360.0;
    glutPostRedisplay ();
}

void reshape(int w, int h)
{
    glViewport (0, 0, (GLsizei) w, (GLsizei) h);
    glMatrixMode (GL_PROJECTION);
    glLoadIdentity ();
    gluPerspective (60.0, (GLfloat) w/(GLfloat) h, 1.0, 20.0);
    glMatrixMode (GL_MODELVIEW);
    glLoadIdentity ();
    gluLookAt (0.0,0.0,5.0, 0.0,0.0,0.0, -0.2,1.0,0.0);
}

void keyboard (unsigned char key, int x, int y)
{
    switch (key) {
        case 'y':
            glutIdleFunc (yearDisplay);
            break;
        case 'n':
            glutIdleFunc (NULL);
            break;
        default:
            break;
    }
}

int main(int argc, char** argv)
{
    glutInit (&argc, argv);
    glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
    glutInitWindowSize (1000, 1000);
    glutInitWindowPosition (100, 100);
    glutCreateWindow ("Animation: solar system");
    init ();
    glutDisplayFunc (display);
    glutReshapeFunc (reshape);
    glutKeyboardFunc (keyboard);
    glutMainLoop();
    return 0;
}
