// Texture Mapping
#include <GL/glut.h>
#include <stdlib.h>
#include <stdio.h>

/* Create checkerboard texture */
#define checkImageWidth 128
#define checkImageHeight 128

static GLubyte checkImage[checkImageHeight][checkImageWidth][4];
static GLuint texName;

void makeCheckImage(void)
{
    int i,j,c;
    
    for (i = 0; i < checkImageHeight; i++) {
        for (j = 0; j < checkImageWidth; j++) {
            c = ((((i&0x8)==0)^((j&0x8))==0))*255;
            checkImage[i][j][0] = (GLubyte) c;
            checkImage[i][j][1] = (GLubyte) c;
            checkImage[i][j][2] = (GLubyte) c;
            checkImage[i][j][3] = (GLubyte) 255;
        }
    }
}

void init(void)
{
    glClearColor (1.0, 1.0, 1.0, 1.0);
    glShadeModel (GL_FLAT);  
    glEnable(GL_DEPTH_TEST);
    
    makeCheckImage();
    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
    
/* Name and create a texture object for a texture image */    
    glGenTextures(1, &texName);
    glBindTexture(GL_TEXTURE_2D, texName);

/* Specify how the texture is to be wrapped and how the colors are to be filtered */    
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER,GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,GL_NEAREST);

/* Specify a single, full-resolution texture map */    
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, checkImageWidth, checkImageHeight,
                0, GL_RGBA, GL_UNSIGNED_BYTE, checkImage);
}
        
void display (void)
{ 
    glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    
/* Turn on texturing; glTexEnvf() sets the drawing mode to GL_DECAL  */    
    glEnable(GL_TEXTURE_2D);
    glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_DECAL);
    
/* Call to the texture object  */    
    glBindTexture(GL_TEXTURE_2D, texName);

/* Texture coordinates are specified along with the vertex coordinates for 2 polygons */    
    glBegin(GL_QUADS);
        glTexCoord2f(0.0,0.0); glVertex3f(-2.0,-1.0,0.0);
        glTexCoord2f(0.0,1.0); glVertex3f(-2.0, 1.0,0.0);
        glTexCoord2f(1.0,1.0); glVertex3f( 0.0, 1.0,0.0);
        glTexCoord2f(1.0,0.0); glVertex3f( 0.0,-1.0,0.0);

        glTexCoord2f(0.0,0.0); glVertex3f(1.0,-1.0,0.0);
        glTexCoord2f(0.0,1.0); glVertex3f(1.0, 1.0,0.0);
        glTexCoord2f(1.0,1.0); glVertex3f(2.41421, 1.0,-1.41421);
        glTexCoord2f(1.0,0.0); glVertex3f(2.41421,-1.0,-1.41421);

        glVertex3f(-2.0,-1.0,0.0);
        glVertex3f(-2.0, 1.0,0.0);
        glVertex3f( 0.0, 1.0,0.0);
        glVertex3f( 0.0,-1.0,0.0);

        glVertex3f(1.0,-1.0,0.0);
        glVertex3f(1.0, 1.0,0.0);
        glVertex3f(2.41421, 1.0,-1.41421);
        glVertex3f(2.41421,-1.0,-1.41421);
    glEnd();
    glFlush();
    glDisable(GL_TEXTURE_2D);		/* Turn off texturing */
}

void reshape(int w, int h)
{
    glViewport (0, 0, (GLsizei) w, (GLsizei) h);
    glMatrixMode (GL_PROJECTION);
    glLoadIdentity ();
    gluPerspective (60.0, (GLfloat) w/(GLfloat) h, 1.0, 30.0);
    glMatrixMode (GL_MODELVIEW);
    glLoadIdentity ();
    glTranslatef(0.0,0.0,-3.6);
}

void keyboard (unsigned char key, int x, int y)
{
    switch (key) {
        case 27:
            exit(0);
            break;
        default:
            break;
    }
}

int main(int argc, char** argv)
{
    glutInit (&argc, argv);
    glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB);
    glutInitWindowSize (1000, 1000);
    glutInitWindowPosition (100, 100);
    glutCreateWindow ("Texture Mapping");
    init ();
    glutDisplayFunc (display);
    glutReshapeFunc (reshape);
    glutKeyboardFunc (keyboard);
    glutMainLoop();
    return 0;
}
